@@ -5,21 +5,7 @@ import numpy as np
 
 from .array_utils import findArrayClosestIndices
 from .system_utils import getfinsnapnum
-
-
-### Constants
-G = 4.301e-9 #km^2 Mpc MSun^-1 s^-2
-
-##### THIS CHANGES WHAT HUBBLE IS
-REDSHIFT=0
-
-#from WMAP
-H0=71 # km /s / Mpc
-OMEGA_0 = 0.27
-OMEGA_B = 0.044
-OMEGA_LAMBDA = 1-OMEGA_0
-HUBBLE=H0*(OMEGA_0*(1+REDSHIFT)**3+OMEGA_LAMBDA)**0.5 #km/s / Mpc
-RHOCRIT = 3*HUBBLE**2./(8*np.pi*G)# 139 Msun / Mpc^3
+from .constants import cm_per_kpc
 
 ### Hernquist/NFW Profiles
 def findCCFromHernquistA(m200,akpc):
@@ -59,10 +45,27 @@ def contained_hernquist_profile(MR,R,a,r):
 
 ### Lookback time (in context of converting stellar ages)
 def convertStellarAges(HubbleParam,Omega0,stellar_tform,Time):
-    """ Assumes a flat cosmology"""
+    """ Assumes a flat cosmology
+
+        a0 = stellar_tform;
+        a2 = All.Time;        
+
+        /* use exact solution for flat universe, ignoring the radiation-dominated epoch [no stars forming then] */
+            /* use simple trap rule integration */
+            a1 = 0.5*(a0+a2);
+            x0 = 1./(a0*hubble_function(a0));
+            x1 = 1./(a1*hubble_function(a1));
+            x2 = 1./(a2*hubble_function(a2));
+            age = (a2-a0)*(x0+4.*x1+x2)/6.;
+        }
+    } 
+    age *= UNIT_TIME_IN_GYR; // convert to absolute Gyr
+    if((age<=1.e-5)||(isnan(age))) {age=1.e-5;}
+    return age;
+}
+    """
     
-    km_per_kpc = 3.086e16
-    UnitTime_in_seconds = km_per_kpc / HubbleParam #/ 1 kms suppressed
+    UnitTime_in_seconds = 1e5*cm_per_kpc / HubbleParam #/ 1 kms suppressed
     UnitTime_in_Megayears = UnitTime_in_seconds/3.1536e13
     
     Hubble_H0_CodeUnits = 3.2407789e-18 * UnitTime_in_seconds 
@@ -83,13 +86,10 @@ def RydenLookbackTime(HubbleParam,Omega_Matter,scale_factors):
     """Compute the lookback time to a (list of) scale factor(s) analytically
         assuming a flat cosmology. """
 
-    km_per_kpc = 3.086e16
-    UnitTime_in_seconds = km_per_kpc / HubbleParam #/ 1 kms suppressed
+    UnitTime_in_seconds = 1e5*cm_per_kpc / HubbleParam #/ 1 kms suppressed
     UnitTime_in_Megayears = UnitTime_in_seconds/3.1536e13
-    
     Hubble_H0_CodeUnits = 3.2407789e-18 * UnitTime_in_seconds
     
-    
     prefactor = 2/3 * 1/np.sqrt(1-Omega_Matter)
     prefactor *= 0.001*UnitTime_in_Megayears/HubbleParam/Hubble_H0_CodeUnits
     
@@ -245,12 +245,12 @@ If you read the halo merger trees (tree*.dat or tree.hdf5) you have:
             try: my_tree[key] = handle[key][:halo_index]
             except: my_tree[key] = handle[key][()]
             
-    return my_tree 
+    return my_tree,treefile
 
 def trace_rockstar(*args,**kwargs):
 
     ## load the merger tree for this halo
-    my_tree = load_rockstar_tree(*args,**kwargs)
+    my_tree,treefile = load_rockstar_tree(*args,**kwargs)
 
     ## follow the progenitor.main.index chain, 
     ##  each main progenitor has its own main progenitor (and a snapshot
@@ -268,7 +268,7 @@ def trace_rockstar(*args,**kwargs):
     halo_traj = my_tree['position'][chain] ## in comoving kpc!
     r200s = my_tree['radius'][chain]
 
-    return snapshots[sort_inds],halo_traj[sort_inds],r200s[sort_inds]
+    return snapshots[sort_inds],halo_traj[sort_inds],r200s[sort_inds],treefile
 
 def fancy_trace_rockstar(snapdir,rockstar_path=None,fancy_trace=True,loud=False):
 
