@@ -16,6 +16,13 @@ from scipy.interpolate import interp1d
 latex_pagewidth=6.9738480697 ## in
 latex_columnwidth=3.32 ## in
 
+from .pfh_colormaps import load_my_custom_color_tables
+try: load_my_custom_color_tables()
+except: pass
+
+try: import palettable
+except: pass #print("palettable colormaps are not installed")
+
 
 """
 try:
@@ -82,7 +89,7 @@ def add_many_to_legend(
 
 
 
-def detach_legend(ax,legend=None,**legend_kwargs):
+def detach_legend(ax,legend=None,transpose=False,**legend_kwargs):
     if legend is None:
         legend = ax.get_legend()
 
@@ -93,9 +100,28 @@ def detach_legend(ax,legend=None,**legend_kwargs):
     lines = legend.get_lines()
     labels = [text.get_text() for text in legend.get_texts()]
 
+    if transpose and 'ncol' in legend_kwargs:
+        ncol = legend_kwargs['ncol']
+        if len(lines)%ncol > 0: to_add = [None]*(ncol-len(lines)%ncol)
+        else: to_add = []
+
+        new_lines = lines+to_add
+        new_lines = np.array(new_lines).reshape(-1,ncol)
+
+        new_lines = new_lines.T.flatten()
+
+        new_labels = labels+to_add
+        new_labels = np.array(new_labels).reshape(-1,ncol)
+        new_labels = new_labels.T.flatten()
+
+        lines = new_lines[new_lines != None]
+        labels = new_labels[new_labels != None]
+        
+
     ## clear out the current legend
     ax.legend([])
-    ax.legend(lines,labels,**legend_kwargs)
+    legend = ax.legend(lines,labels,**legend_kwargs)
+    return legend
         
 def add_to_legend(
     ax,
@@ -228,6 +254,20 @@ def plotEllipse(
                 lw=lw,
                 **kwargs))
 
+def plotRing(ax,x0,y0,inner_radius,outer_radius,**kwargs):
+
+    n, radii = 50, np.array([inner_radius,outer_radius])
+    theta = np.linspace(0, 2*np.pi, n, endpoint=True)
+    xs = np.outer(radii, np.cos(theta))
+    ys = np.outer(radii, np.sin(theta))
+
+    # in order to have a closed area, the circles
+    # should be traversed in opposite directions
+    xs[1,:] = xs[1,::-1] + x0
+    ys[1,:] = ys[1,::-1] + y0
+
+    return ax.fill(np.ravel(xs), np.ravel(ys),**kwargs)
+
 def addColorbar(
     ax,cmap,
     vmin,vmax,
@@ -459,9 +499,9 @@ def talkifyAxes(axs,lw=2,labelsize=24,ticklabelsize=16):
               ax.spines[axis].set_linewidth(lw)
         ax.xaxis.label.set_size(labelsize)
         ax.yaxis.label.set_size(labelsize)
-        if ax.is_first_col():
+        if ax.get_subplotspec().is_first_col():
             ax.yaxis.set_ticklabels(ax.yaxis.get_ticklabels(),fontsize=ticklabelsize)
-        if ax.is_last_row():
+        if ax.get_subplotspec().is_last_row():
             ax.xaxis.set_ticklabels(ax.xaxis.get_ticklabels(),fontsize=ticklabelsize)
 
 def slackifyAxes(ax,width=8,height=6):
@@ -483,6 +523,22 @@ def my_log_formatter(x,y):
 
 my_log_ticker = matplotlib.ticker.FuncFormatter(my_log_formatter)
 
+
+## from https://stackoverflow.com/questions/41597177/get-aspect-ratio-of-axes
+from operator import sub
+def get_aspect(ax):
+    # Total figure size
+    figW, figH = ax.get_figure().get_size_inches()
+    # Axis size on figure
+    _, _, w, h = ax.get_position().bounds
+    # Ratio of display units
+    disp_ratio = (figH * h) / (figW * w)
+    # Ratio of data units
+    # Negative over negative because of the order of subtraction
+    data_ratio = sub(*ax.get_ylim()) / sub(*ax.get_xlim())
+
+    return disp_ratio / data_ratio
+
 ## from https://gist.github.com/benmaier/31f5fa109cf8fae077bde3d2d68a3883
 def add_curve_label(
     ax,
@@ -607,23 +663,24 @@ def bufferAxesLabels(
     for col_i in range(ncols):
         this_col = axss[:,col_i]
         for ax in this_col:
-            if ylabels and not ax.is_first_col():
+            if ylabels and not ax.get_subplotspec().is_first_col():
                 ax.set_ylabel('')
-            try:
-                xticks = ax.get_xticklabels()
-                xtick_strings = np.array([xtick.get_text() for xtick in xticks])
-                if len(xticks) == 0:
-                    continue
+            if ncols > 1:
+                try:
+                    xticks = ax.get_xticklabels()
+                    xtick_strings = np.array([xtick.get_text() for xtick in xticks])
+                    if len(xticks) == 0:
+                        continue
 
-                ##  change the first tick
-                if not ax.is_first_col():
-                    xticks[0].set_horizontalalignment('left')
-                ## if we're in the right most 
-                ##  column we don't need to change the last tick
-                #if col_i != (ncols-1):
-                xticks[-1].set_horizontalalignment('right')
-            except IndexError:
-                pass ## this can fail if share_x = True
+                    ##  change the first tick
+                    if not ax.get_subplotspec().is_first_col():
+                        xticks[0].set_horizontalalignment('left')
+                    ## if we're in the right most 
+                    ##  column we don't need to change the last tick
+                    #if col_i != (ncols-1):
+                    xticks[-1].set_horizontalalignment('right')
+                except IndexError:
+                    pass ## this can fail if share_x = True
 
     for ax in axss.flatten():
         if xlabels:
@@ -634,11 +691,11 @@ def bufferAxesLabels(
             if len(yticks) == 0:
                 continue
             ##  need to mess with the top tick
-            if not ax.is_first_row():
+            if not ax.get_subplotspec().is_first_row():
                 yticks[-1].set_verticalalignment('top')
             ## if we're in the last row we 
             ##  don't need to mess with the bottom tick
-            if not ax.is_last_row():
+            if not ax.get_subplotspec().is_last_row():
                 yticks[0].set_verticalalignment('bottom')
         except IndexError as e:
             pass ## this can fail if share_y = True
@@ -1111,6 +1168,12 @@ def ffmpeg_frames(
             print(cmd)
             os.system(cmd)
 
+            src = os.path.join(outdir,frame_head+extension)
+            dst = os.path.join(os.getcwd(),os.path.basename(dst))
+            cmd = 'cp %s %s'%(src,dst)
+            print(cmd)
+            os.system(cmd)
+
 
 def place_text(ax,percentage):
     ylow,yhigh = ax.get_ylim()
@@ -1127,14 +1190,6 @@ def place_text(ax,percentage):
         foo = 10**foo
     return foo
 
-try: import palettable
-except: print("palettable colormaps are not installed")
-try:
-    from pfh_colormaps import load_my_custom_color_tables
-    load_my_custom_color_tables()
-except:
-    print("don't have phil's colormaps")
-
 def get_cmap(cmap_name):
     try: cmap = plt.get_cmap(cmap_name)
     except:
@@ -1177,4 +1232,29 @@ def set_size(fig, size, eps=1e-2, give_up=2, min_size_in=0.25):
         if len(deltas) > give_up and sorted(deltas[-give_up:]) == deltas[-give_up:]:
             return False
         if set_width < min_size_in or set_height < min_size_in:
-            return False
\ No newline at end of file
+            return False
+    
+def clean_savefig(fig,figname,width,height,plotdir=None,savefig_flag=True):
+
+    if plotdir is None: plotdir = os.getcwd()
+    elif plotdir[0] != os.sep: plotdir = os.path.abspath(os.path.join(os.getcwd(),plotdir))
+
+    set_size(fig,(width,height))
+
+    """
+    fig.subplots_adjust(hspace=0,wspace=0)#,left=0,right=1,bottom=0,top=1)
+    if column == 1:
+        set_size(fig,(latex_columnwidth,latex_columnwidth*h_scale_factor))
+    elif column == 2:
+        set_size(fig,(latex_pagewidth,latex_pagewidth*h_scale_factor))
+    else:
+        raise ValueError("Column %d should be 1 or 2")
+    """
+
+    fig.set_facecolor('white')
+
+    print("saving:",plotdir,figname)
+    if not savefig_flag:
+        print('skipping saving')
+        return
+    fig.savefig(os.path.join(plotdir,figname),bbox_inches='tight')
